package ru.ulstu.extractor.ts;

import org.apache.commons.lang3.time.DateUtils;
import ru.ulstu.extractor.model.TimeSeries;
import ru.ulstu.extractor.model.TimeSeriesValue;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Класс для регулировки дискретности временного ряда.
 * Можно создать временной ряд с точками с секундными отметками, затем суммировать значения,
 * применив одно из значений TimeSeriesInterval
 */
public class TimeSeriesDateMapper {

    public TimeSeries mapTimeSeriesToInterval(TimeSeriesInterval timeSeriesInterval, TimeSeries timeSeries) {
        List<TimeSeriesValue> trimmedTimeSeriesValues = timeSeries.getValues()
                .stream()
                .map(timeSeriesValue -> new TimeSeriesValue(trimTo(timeSeriesInterval, timeSeriesValue.getDate()),
                        timeSeriesValue.getValue()))
                .collect(Collectors.toList());
        Map<Date, Integer> groupedTimeSeriesValues = trimmedTimeSeriesValues
                .stream()
                .collect(Collectors.groupingBy(TimeSeriesValue::getDate,
                        Collectors.summingInt(TimeSeriesValue::getValue)));

        return new TimeSeries(timeSeries.getName(), groupedTimeSeriesValues.entrySet()
                .stream()
                .map(e -> new TimeSeriesValue(e.getKey(), e.getValue()))
                .collect(Collectors.toList()));
    }

    private Date trimTo(TimeSeriesInterval timeSeriesInterval, Date date) {
        return DateUtils.truncate(date, timeSeriesInterval.calendarField);
    }

    /**
     * Интервальность временного ряда при преобразовании
     */
    public enum TimeSeriesInterval {
        SECOND(Calendar.SECOND),
        MINUTE(Calendar.MINUTE),
        HOUR(Calendar.HOUR_OF_DAY),
        DAY(Calendar.DAY_OF_MONTH),
        WEEK(Calendar.WEEK_OF_MONTH),
        MONTH(Calendar.MONTH),
        YEAR(Calendar.YEAR);

        private final int calendarField;

        TimeSeriesInterval(int calendarField) {
            this.calendarField = calendarField;
        }
    }
}
